<span
    class="badge rounded-pill mb-0 fw-normal
        {% block badge_class %}{% endblock %}
        {{ badge_class|default:'fs--1' }}
    "
>
    {% block badge_text %}{% endblock %}
    {{ badge_text }}

    {% if x_badge_text %}
        <span x-text="{{ x_badge_text }}"></span>
    {% endif %}
</span>
