{% extends 'django_spire/card/title_card.html' %}

{% block card_title %}
    Files
{% endblock %}

{% block card_button %}
    {% url 'core_file:demo_file_form' demo_pk=demo.pk pk=0 as new_demo_file_url %}
    {% include 'django_spire/button/secondary_outlined_button.html' with button_href=new_demo_file_url button_logo_only='bi bi-plus' %}
{% endblock %}

{% block card_title_content %}
    {% if queryset %}
        {% for file in file_list %}
            {% include 'django_spire/file/item/file_item.html' with delete_url=delete_url %}
        {% endfor %}
    {% else %}
        {% include 'django_spire/item/no_data_item.html' %}
    {% endif %}
{% endblock %}
