<div class="d-flex align-items-center justify-content-center" style="width: 35px;">
    {% block choice_checkmark %}
        <template x-if="value === choice[0]">
            {% include 'django_spire/form/field/element/select_checkmark_element.html' %}
        </template>

        <template x-if="value != choice[0]">
            <span class="ms-3 ps-2"></span>
        </template>
    {% endblock %}
</div>

<div class="d-flex align-items-center">
    <span class="text-truncate" x-text="choice[1]"></span>
</div>
