.full-page-content {
    width: 100%;
}

.main-content-with-panels {
    transition: margin-left 0.3s ease, margin-right 0.3s ease;
}

.main-content-with-panels.with-left-panel {
    margin-left: calc(var(--app-side-panel-width) * 0.5);
}

@media (min-width: 992px) {
    .full-page-content {
        width: calc(100% - var(--app-side-navigation-width));
    }
}
