[data-theme="dark"] {
    --app-primary: #59c2ff;
    --app-primary-soft: #0f232e;
    --app-primary-dark: #39bae6;

    --app-secondary: #5c6773;
    --app-secondary-soft: #262d35;
    --app-secondary-dark: #4d5562;

    --app-accent: #ff8f40;
    --app-accent-soft: #3d2414;
    --app-accent-dark: #ff7733;

    --app-success: #95e6cb;
    --app-success-soft: #1a2e2a;
    --app-success-dark: #7fd4b4;

    --app-warning: #ffb454;
    --app-warning-soft: #3d2f14;
    --app-warning-dark: #ffa940;

    --app-danger: #f07178;
    --app-danger-soft: #3d2426;
    --app-danger-dark: #ed5a61;

    --app-layer-one: #0d1017;
    --app-layer-two: #1f2430;
    --app-layer-three: #272d38;
    --app-layer-four: #3e4b59;

    --app-alt-layer-one: #0a0e14;
    --app-alt-layer-two: #1a1f29;
    --app-alt-layer-three: #232834;
    --app-alt-layer-four: #343a46;

    --app-card-header: #1f2430;

    --app-default-attribute-title-color: #bfbdb6;
    --app-default-button-text-color: #0f1419;
    --app-default-text-color: #bfbdb6;
    --app-default-alt-text-color: #acaaa2;
    --app-default-link-color: #59c2ff;
    --app-default-link-hover-color: #ffd580;

    --app-inverted-text-color: #0f1419;
    --app-inverted-alt-text-color: #262d35;

    --app-side-navigation-bg-color: #0d1017;
    --app-side-navigation-text-color: #bfbdb6;
    --app-side-navigation-link-color: #bfbdb6;
    --app-side-navigation-link-hover-color: #59c2ff;

    --app-top-navigation-bg-color: #0d1017;
    --app-top-navigation-text-color: #bfbdb6;
    --app-top-navigation-link-color: #59c2ff;
    --app-top-navigation-link-hover-color: #ffd580;

    --app-footer-bg-color: #0d1017;
    --app-footer-text-color: #bfbdb6;
    --app-footer-link-color: #59c2ff;
    --app-footer-link-hover-color: #ffd580;

    --app-table-row-color: #0f1419;
    --app-table-row-alt-color: #1f2430;
    --app-table-row-hover-color: #272d38;

    --bs-border-color: #3e4b59;

    --app-shadow-light: rgba(0, 0, 0, 0.3);
    --app-shadow-medium: rgba(0, 0, 0, 0.5);
    --app-shadow-heavy: rgba(0, 0, 0, 0.7);
    --app-shadow-navigation: rgba(0, 0, 0, 0.9);

    --app-border-translucent-light: rgba(62, 75, 89, 0.3);
    --app-focus-ring-primary: rgba(89, 194, 255, 0.25);
}
