{% extends 'django_spire/form/field/base_field.html' %}

{% block field_content %}
    <textarea
            x-ref="glue_field"
            type="text"
            class="form-control"
            x-model.debounce.500ms="value"
            rows="{% if rows %}{{ rows }}{% else %}3{% endif %}"
    ></textarea>
{% endblock %}
