{% extends 'django_spire/form/field/base_field.html' %}

{% block field_content %}
    <div class="form-group">
        <template x-for="choice in glue_field.choices" :key="choice[0]">
            <div class="form-check">
                <input
                        type="radio"
                        class="form-check-input"
                        :id="glue_field.id + '_' + choice[0]"
                        :name="glue_field.name"
                        :value="choice[0]"
                        :checked="value === choice[0]"
                        @change="this.value =choice[0]"
                >
                <label
                        class="form-check-label"
                        :for="glue_field.id + '_' + choice[0]"
                        x-text="choice[1]"
                ></label>
            </div>
        </template>
    </div>
{% endblock %}