{% extends 'django_spire/page/page.html' %}

{% load static %}

{% block page_content %}
    <div class="row justify-content-center mx-auto mt-5">
        <div class="col-11 col-md-6 col-lg-4 bg-app-layer-one">
            <div class="px-4 py-5">
                {% block authentication_page_top_content %}
                    <div class="text-center pb-5">
                        <a href="/">
                            {% block authentication_page_img %}
                                <img height="120px" class="ms-1" src="{% static 'django_spire/img/django_spire.svg' %}"
                                     alt="">
                            {% endblock %}
                        </a>
                    </div>
                {% endblock %}
                {% block authentication_page_content %}

                {% endblock %}
            </div>
        </div>
    </div>
{% endblock %}

