# Generated by Django 5.2.7 on 2025-10-25 15:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_knowledge', '0005_entryversionblock__tunes_data_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entryversionblock',
            name='type',
            field=models.CharField(choices=[('text', 'Text'), ('heading', 'Heading'), ('list', 'List')], default='text', max_length=32),
        ),
    ]
