{% extends 'django_spire/item/item.html' %}

{% load spire_core_tags %}

{% block item_title_col %} d-none{% endblock %}

{% block item_row_content %}
    <div class="col-8">
        {% url 'django_spire:auth:user:page:detail' pk=user.pk as detail_url %}
        {% include 'django_spire/element/attribute_element.html' with attribute_title='Name' attribute_value=user.get_full_name attribute_href=detail_url %}
    </div>

    <div class="col-4">
        {% include 'django_spire/element/attribute_element.html' with attribute_title='Status' %}
        {% include 'django_spire/auth/group/element/is_active_badge.html' with is_active=user.is_active %}
    </div>
{% endblock %}

{% block item_button %}
    {% if perms.django_spire_auth_group.delete_authgroup %}
        {% url 'django_spire:auth:group:form:user_remove' group_pk=group.pk pk=user.pk as delete_url %}
        {% include 'django_spire/dropdown/ellipsis_dropdown.html' with delete_url_name='Remove' delete_url=delete_url %}
    {% endif %}
{% endblock %}
