.fs-print-1 {
    font-size: 1.0rem !important;
}

.fs-print-2 {
    font-size: 0.90rem !important;
}

.fs-print-3 {
    font-size: 0.70rem !important;
}

@media print {

    .fs-print-1 {
        font-size: .80rem !important;
    }

    .fs-print-2 {
        font-size: .70rem !important;
    }

    .fs-print-3 {
        font-size: .55rem !important;
    }

    .hide-on-print {
        display: none !important;
    }

    @page {
        margin: 1cm !important;
    }

    div {
        page-break-inside: auto !important;
    }

    .page-break-before {
        page-break-before: always !important;
        display: block;
        height: 1px;
        clear: both;
    }

    .table-hover > tbody > tr:hover > * {
        --bs-table-bg-state: var(--app-table-row-color) !important;
        background-color: var(--app-table-row-color) !important;
        color: var(--app-default-text-color) !important;
    }

}

.table-print {
    font-family: monospace !important;
}
