{% extends 'django_spire/form/field/base_field.html' %}

{% block field_content %}
    <input
            {% if input_type %}type="{{ input_type }}"{% else %}type="number"{% endif %}
            class="form-control"
            x-ref="glue_field"
            x-model="value"
            :id="glue_field.id"
    >
{% endblock %}
