<div
    class="card ms-0 shadow-sm rounded-3 bg-app-layer-two {% block card_class %}{% endblock %}"
    style="{% block card_style %}{% endblock %}"
>
    <div class="h-100 {% block card_padding %}p-3{% endblock %}" style="min-height: 0; {% block card_body_style %}{% endblock %}">
        {% block card_content %}{% endblock %}
    </div>

    {% block card_footer %}{% endblock %}
</div>
