{% extends 'django_spire/modal/title_modal.html' %}

{% load comment_tags %}

{% block modal_trigger %}
    <div class="px-4 p-1">Reply</div>
{% endblock %}

{% block modal_title %}
    Reply to Comment
{% endblock %}

{% block modal_title_content %}
    <div class="bg-layer-two border rounded p-3 my-2">
        <div class="fs--1 text-secondary">{{ comment.user.get_full_name }}</div>
        <div>{{ comment.information }}</div>
    </div>
    <div class="mx-4">
        {% comment_form related_obj request.get_full_path request.user parent=comment user_list=comment_user_list as comment_form %}
        <div class="h6">Reply*</div>
        {{ comment_form }}
    </div>
{% endblock %}
