# Generated by Django 5.2 on 2025-05-06 21:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_ai_chat', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='chatmessage',
            name='content',
        ),
        migrations.AddField(
            model_name='chatmessage',
            name='_content',
            field=models.JSONField(default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='chatmessage',
            name='_content_intel_class',
            field=models.CharField(default=' ', max_length=64),
            preserve_default=False,
        ),
    ]
