{% extends 'django_spire/auth/page/auth_page.html' %}

{% block authentication_page_content %}
    {% include 'django_spire/auth/form/login_form.html' %}

    <div class="text-center">
        <a href="{% url 'django_spire:auth:admin:password_reset' %}"
            class="fs--1 text-app-secondary cursor-pointer">
            Forgot my password
        </a>
    </div>
    {% include 'django_spire/auth/element/android_and_chrome_app_install_element.html' %}
    {% include 'django_spire/auth/element/ios_app_install_element.html' %}
{% endblock %}
