{% extends 'django_spire/base/base.html' %}

{% block base_body_content %}
    <div class="row mt-6">
        <div class="col text-center mb-3 h1 text-app-warning">
            <i class="bi bi-cone-striped"></i>
        </div>
    </div>

    <div class="row mt-4">
        <div class="col text-center mb-3">
            <div class="h4 text-app-primary mb-3 fw-bold">
                Maintenance Mode
            </div>
            <p>
            The site is currently in maintenance mode for some critical work and will be back up shortly.
            </p>
        </div>
    </div>

{% endblock %}
