{% extends 'django_spire/page/full_page.html' %}

{% block full_page_content %}
{#    <div class="row mb-3">#}
{#        <div class="col">#}
{#            {% include 'django_spire/auth/group/card/detail_card.html' %}#}
{#        </div>#}
{#    </div>#}

    <div class="row g-lg-3">
        <div class="col-lg-8 mb-lg-0 mb-3">
            {% include 'django_spire/auth/group/card/group_permission_list_card.html' %}
        </div>
        <div class="col-lg-4">
            <div class="mb-3">
                {% include 'django_spire/auth/group/tab/user_list_card_tabs.html'%}
            </div>

            {% include 'django_spire/activity/card/list_card.html' with queryset=group.activity_log.all %}
        </div>
    </div>
{% endblock %}
