# Generated by Django 5.2.10 on 2026-01-08 20:41

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ReportRun',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('report_key_stack', models.TextField()),
                ('datetime', models.DateTimeField(default=django.utils.timezone.now)),
            ],
            options={
                'verbose_name': 'Report Run',
                'verbose_name_plural': 'Report Runs',
                'db_table': 'django_spire_metric_report_run',
            },
        ),
    ]
