{% extends 'django_spire/ai/chat/message/message.html' %}

{% load spire_ai_chat_tags %}

{% block message_content %}
    <div class="ai-message-content">
        {{ message_intel.text|render_markdown }}
    </div>
{% endblock %}
